//
//  MemoryLeaker.swift
//  TestBsonLeak
//
//  Created by Nikolay Dzhulay on 03/01/2021.
//

import Foundation
import MongoSwift

final class MemoryLeaker {
    static func leak() throws {

        for _ in 0...200000 {
            let bsonDocument = try! BSONDocument(fromJSON: .bsonJson)
            _ = try Installation.create(from: bsonDocument)
        }
        print("Completed massive decoding")
    }

    static func nativeDecoding() throws {
        let jsonData = String.usualJson.data(using: .utf8)!

        for _ in 0...200000 {
            let decoder = JSONDecoder()
            decoder.dateDecodingStrategy = .secondsSince1970
            _ = try decoder.decode(Installation.self, from: jsonData)
        }
        print("Completed massive decoding")
    }
}

extension Installation {
    static func create(from document: BSONDocument) throws -> Installation {
        let decoder = BSONDecoder()
        decoder.dateDecodingStrategy = .bsonDateTime
        return try decoder.decode(Self.self, from: document)
    }
}


private extension String {
    static let bsonJson: String =
    """
    {
      "_id": {
        "$oid": "5f31b5f8efc5d92cdc440e1b"
      },
      "id": "0F45AF1F-2589-4636-94DD-123E749251E5_2019",
      "appInfo": {
        "appIdentifier": "com.some",
        "appVersion": "3.4.6.1",
        "isDebug": true
      },
      "createdAt": {
        "$date": "2020-08-10T21:02:48.000Z"
      },
      "deviceInfo": {
        "osVersion": "14.0.1",
        "deviceType": "ios",
        "device": "iPhone12,5",
        "localeIdentifier": "en_GB",
        "timeZone": "Europe/London"
      },
      "installationInfo": {
        "deviceToken": "CB99CB45E4868F3C1FAE799FF917FDF441464F3F0F623A86C120D26F8DBD06D8",
        "tags": [],
        "languageCode": "en"
      },
      "partitionKey": "2019",
      "createdAtSeconds": {
        "$numberLong": "1597093368"
      }
    }
    """


    static let usualJson: String =
    """
    {
      "id": "0F45AF1F-2589-4636-94DD-123E749251E5_2019",
      "appInfo": {
        "appIdentifier": "com.some",
        "appVersion": "3.4.6.1",
        "isDebug": true
      },
      "createdAt": 1597093368,
      "deviceInfo": {
        "osVersion": "14.0.1",
        "deviceType": "ios",
        "device": "iPhone12,5",
        "localeIdentifier": "en_GB",
        "timeZone": "Europe/London"
      },
      "installationInfo": {
        "deviceToken": "CB99CB45E4868F3C1FAE799FF917FDF441464F3F0F623A86C120D26F8DBD06D8",
        "tags": [],
        "languageCode": "en"
      },
      "partitionKey": "2019",
      "createdAtSeconds": 1597093368
    }
    """
}
